#!/bin/python2

import sys
import os

def load_prv(file_):
    trace = open(file_, 'r')
    mapa = {}
    while True:
        line = trace.readline()
        if line == '' : break
        line = line[:-1] # remove the \n char
        line = line.split(':')
        key = ""
        value = [ 0 ] * 30
        value [1] = sys.maxint
        value [2] = sys.maxint
        ID = -1
        # There must be a better way:
        for index, val in enumerate(line):
            # Last eight values have information, the previous are the key.
            if val == '49000001' :
                # We arrived to the Simulator values
                for k, v in zip(line[index::2], line[index+1::2]) :
                    value[int(k) - 49000000] = int(v)
                break
            else:
                # Still in the header
                key = key + val + ":"
        key = key[:-1] # remove last :
        if not mapa.has_key(key) : mapa [key] = value
        else :
            mapa[key][1] = min(mapa[key][1], value[1])
            mapa[key][2] = min(mapa[key][2], value[2])
            mapa[key][3] = max(mapa[key][3], value[3])
            for cache_level in range(11, len(value)):
                mapa[key][cache_level] = mapa[key][cache_level] + value[cache_level]
    trace.close()
    return mapa

def load_pcf(file_):

    mapa = {}
    trace = open(file_, 'r')
    reading_event = False
    reading_values = False
    key = ""
    while True:
        line = trace.readline()
        if line == '' : break
        line = line[:-1] # remove the \n char
        if line == "EVENT_TYPE" :  reading_event = True
        elif line == "VALUES" : reading_values = True
        elif reading_event :
            line = " ".join(line.split())
            line = line.split (' ')
            key = line[1]
            mapa [key] = []
            mapa [key].append(line[2])
            reading_event = False
        elif reading_values :
            if line == "" : reading_values = False
            else:
                if key == "" : print ("Error : Key = " + key + " line = " + line)
                line = line.split (' ')
                mapa [key].append(line[1])
    return mapa

def load_lat(file_):
    mapa = {}
    fp_inst = set()
    alu_inst = set()
    non_alu_inst = set()
    if not os.path.exists(file_) : return mapa, fp_inst, alu_inst, non_alu_inst
    trace = open(file_, 'r')
    reading_alu_fp = False
    reading_alu_non_fp = False
    reading_non_alu = False
    while True:
        line = trace.readline()
        if line == '' : break
        line = line[:-1] # remove the \n char
        if  line == "ALU_FP_INST" :
            reading_alu_fp = True
            reading_alu_non_fp = False
            reading_non_alu = False
        elif  line == "ALU_NON_FP_INST" :
            reading_alu_non_fp = True
            reading_alu_fp = False
            reading_non_alu = False
        elif  line == "NON_ALU_INST" :
            reading_non_alu = True
            reading_alu_non_fp = False
            reading_alu_fp = False
        elif reading_alu_fp :
            if line == '' :
                reading_alu_fp = False
                continue
            line = line.split(' ')
            fp_inst.add(line[0])
            mapa[line[0]] = line[-1]
        elif reading_alu_non_fp :
            if line == '' :
                reading_alu_non_fp = False
                continue
            line = line.split(' ')
            alu_inst.add(line[0])
            mapa[line[0]] = line[-1]
        elif reading_non_alu :
            if line == '' :
                reading_non_alu = False
                continue
            line = line.split(' ')
            non_alu_inst.add(line[0])
            mapa[line[0]] = line[-1]
    return mapa, fp_inst, alu_inst, non_alu_inst


def get_instr(line, diccionari):
    line = line.split(':')
    for i in xrange(0, len(line), 2):
        if diccionari.has_key(line[i]) and diccionari[line[i]][0] == "Instruction" :
            return diccionari[line[i]][int(line[i+1])]
    return ""

def get_granted_vector_length(line):
    line = line.split(':')
    for index, val in enumerate(line):
        if val == "47000019": return line[index + 1]
    return ""


sim_file = "/home/martit/projects/simulations/logs/riscv/trace_output.prv"
orig_file = "/home/martit/projects/traces/sources/prv/SpMV_packed/vehave.25226"
latency_file = "/home/martit/projects/simulations/conf/vector_100.lat"
vector_unit_throughput = 512 #bits
inorder_commit = 0
if len(sys.argv) != 6:
    print ("usage: " + sys.argv[0] + " <sim_file> <orig_file> <latency_file> <vector_unit_throughput> <inorder_commit>")
    print ("Example: " + sys.argv[0] + " " + sim_file + " " + orig_file + " " + latency_file + " " + vector_unit_throughput+ " " + inorder_commit)
    exit(-1)
else:
    sim_file = sys.argv[1]
    orig_file = sys.argv[2]
    latency_file = sys.argv[3]
    vector_unit_throughput = int(sys.argv[4])
    inorder_commit = int(sys.argv[5])

trace = open(orig_file + '.prv', 'r')
mapa = load_prv(sim_file)
diccionari = load_pcf(orig_file + '.pcf')
latency_map_, fp_ins, alu_ins, non_alu_ins = load_lat(latency_file)
line = trace.readline()
previous_memory_operation_rob_latency = 0
issue_cycle = 0
commit_cycle = 0

# We print the header:
print (line[:-1])

while True:
    # Read the line:
    key = trace.readline()
    # If EOF, break
    if key == '' : break
    # Remove end of line
    key = key[:-1]
    if not mapa.has_key(key) :
        # We do not have information for that instruction
        if get_instr(key, diccionari) == "vsetvli" :
            rob_latency = 0
        else :
            rob_latency =  previous_memory_operation_rob_latency
        #  8x to be transformed to bits
        if get_instr(key, diccionari) in fp_ins :
            granted_vector_length = 1 + ((int(get_granted_vector_length(key)) << 3) / (vector_unit_throughput >> 3))
            latency = rob_latency + (int(latency_map_[get_instr(key, diccionari)]) * granted_vector_length)
        elif get_instr(key, diccionari) in alu_ins :
            granted_vector_length = int(get_granted_vector_length(key))
            latency = rob_latency + (int(latency_map_[get_instr(key, diccionari)]) * granted_vector_length)
        elif get_instr(key, diccionari) in non_alu_ins :
            latency = rob_latency + int(latency_map_[get_instr(key, diccionari)])
        else :
            if latency_map_.has_key(get_instr(key, diccionari)) :
                print ("Error assigning latency 1 to " + get_instr(key, diccionari) + " which has a latency of " + str(latency_map_[get_instr(key, diccionari)]))
                exit(-1)
            latency = rob_latency + 1

        if (commit_cycle >= (issue_cycle + latency)) and (inorder_commit == 1):
            latency = commit_cycle - issue_cycle + 1
        commit_cycle = issue_cycle + latency
        line = key + ":" + "49000000:" + str(latency)
        line = line + ":" + "49000001:" + str(rob_latency)
        line = line + ":" + "49000003:" + str(commit_cycle)
    else :
        issue_cycle = mapa[key][1]
        rob_latency = (mapa[key][2] - mapa[key][1])
        execution_latency = (mapa[key][3] - mapa[key][2])
        if (commit_cycle >= (issue_cycle + rob_latency + execution_latency)) and (inorder_commit == 1) :
            execution_latency = commit_cycle - issue_cycle - rob_latency + 1
        latency = rob_latency + execution_latency
        commit_cycle = issue_cycle + latency
        line = key + ":" + "49000000:" + str(latency)
        line = line + ":" + "49000001:" + str(rob_latency)
        line = line + ":" + "49000002:" + str(execution_latency)
        line = line + ":" + "49000003:" + str(commit_cycle)
        is_memory = True
#        is_memory = False
#        for cache_level in range(11, 15):
#            if mapa[key][cache_level] > 0 :
#                is_memory = True
#        print 'memory? ', is_memory, mapa[key][11], mapa[key][12], mapa[key][13], mapa[key][14], mapa[key][15]
        if is_memory :
            for cache_level in range(11, 15):
                line = line + ":" + str(49000000 + cache_level) + ":" + str(mapa[key][cache_level])
        line = line + ":" +  str(49000021) + ":" + str(mapa[key][21])

        previous_memory_operation_rob_latency = rob_latency
    print (line)

